<?php
namespace Drupal\popsu_migrate\Plugin\migrate\source;

use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\migrate\Row;
use Drupal\migrate_drupal\Plugin\migrate\source\d7\FieldableEntity;
use Drupal\Core\Database\Query\SelectInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandler;
use Drupal\Core\State\StateInterface;
use Drupal\migrate\Plugin\MigrationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drush\Drush;
use Drupal\paragraphs\Entity\Paragraph;

// use Drupal\popsu_migrate\Plugin\migrate\source\D7Programme;

/**
 * Drupal 7 node source from database.
 *
 * @MigrateSource(
 *   id = "d7_node_theme",
 *   source_module = "node"
 * )
 */
class D7NodeTheme extends FieldableEntity {
  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;


  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration, StateInterface $state, EntityTypeManagerInterface $entity_manager, ModuleHandlerInterface $module_handler,LoggerChannelFactoryInterface $loggerFactory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $migration, $state, $entity_manager);
    $this->moduleHandler = $module_handler;
    $this->logger = $loggerFactory->get('popsu_migration');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration = NULL) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $migration,
      $container->get('state'),
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
      $container->get('logger.factory')
    );
  }

  /**
   * The join options between the node and the node_revisions table.
   */
  const JOIN = 'n.vid = nr.vid';

  /**
   * {@inheritdoc}
   */
  public function query() {
    // Select node in its last revision.
    $query = $this->select('node_revision', 'nr')
      ->fields('n', [
        'nid',
        'type',
        'uid',
        'language',
        'status',
        'created',
        'changed',
      ])
      ->fields('nr', [
        'vid',
        'title',
        'log',
        'timestamp',
      ])
      ->orderBy('changed');

    $query->addField('n', 'uid', 'node_uid');
    $query->addField('nr', 'uid', 'revision_uid');
    $query->innerJoin('node', 'n', static::JOIN);

    $query->condition('n.type', ['popsu_theme_local', 'popsu_theme_trans', 'popsu_theme_europe'], 'IN');

    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareRow(Row $row) {
    $nid = $row->getSourceProperty('nid');
    $vid = $row->getSourceProperty('vid');
    $type = $row->getSourceProperty('type');
    $title = $row->getSourceProperty('title');
    Drush::output()->writeln('- - - - ' . $title . '- - - - ');

    // Get Field API field values.
    foreach ($this->getFields('node', $type) as $field_name => $field) {
      $field_values = $this->getFieldValues('node', $field_name, $nid, $vid, NULL);
      $row->setSourceProperty($field_name, $field_values);
      switch ($field_name) {
        case 'field_popsu_themloc_popsu':
          $field_programme = $field_values;
          break;
        case 'field_popsu_themtrans_popsu':
          $field_programme = $field_values;
          break;
        case 'field_popsu_themeur_popsu':
          $field_programme = $field_values;
          break;
        case 'field_popsu_themloc_imageintro':
          $field_images = $field_values;
          break;
        case 'field_popsu_themtrans_imag_illus':
          $field_images = $field_values;
          break;
      }
    }

    if ($field_programme) {
      // Drush::output()->writeln(dump($field_programme));
      // get the nid of popsu_special from the tid of popsu for migration lookup
      // Select node in its last revision.
      $query = $this->select('node_revision', 'nr')
        ->fields('n', ['nid','type','vid'])
        ->fields('nr', ['vid']);
      $query->innerJoin('node', 'n', static::JOIN);
      $query->condition('n.type', 'popsu_special');
      # field_popsu_special_typetaxo
      # filter to get the right special type (colloques, publication, etc)
      $query->leftJoin('field_revision_field_popsu_special_typetaxo', 'ff', 'ff.revision_id = n.vid');
      $query->fields('ff', ['field_popsu_special_typetaxo_tid']);
      $query->condition('ff.field_popsu_special_typetaxo_tid', 31);
      # field_popsu_special_popsu
      # filter to get right popsu (popsu 1, popsu europe, etc)
      $query->leftJoin('field_revision_field_popsu_special_popsu', 'sp', 'sp.revision_id = n.vid');
      $query->fields('sp', ['field_popsu_special_popsu_tid']);
      $query->condition('sp.field_popsu_special_popsu_tid', $field_programme[0]['tid']);

      $prog = $query->execute()->fetchAll();
      // Drush::output()->writeln(dump($prog));

      $row->setSourceProperty('programme', $prog[0]['nid']);
    }

    # Images
    if ($field_images) {
      // Drush::output()->writeln(dump($field_images));
      $row->setSourceProperty('images', $field_images);
    }

    # Themes liés
    // merge the two ppossible field source
    $field_themes_lies = array();
    $field_projets_lies = array();
    if ($field_popsu_themloc_lies = $row->getSourceProperty('field_popsu_themloc_lies')) {
      // Drush::output()->writeln(dump($field_popsu_themloc_lies));
      foreach ($field_popsu_themloc_lies as $key => $value) {
        // check if theme
        $q = $this->select('node', 'n')
          ->fields('n', ['nid','type'])
          ->condition('n.nid', $value['nid'])
          ->condition('n.type', ['popsu_theme_local', 'popsu_projet'], 'IN');
        $res = $q->execute()->fetchAll();
        if (!empty($res)) {
          // Drush::output()->writeln(dump($res));
          switch ($res[0]['type']) {
            case 'popsu_theme_local':
              $field_themes_lies[] = $value;
              break;
            case 'popsu_projet':
              $field_projets_lies[] = $value;
              break;
          }
        }
      }
    }
    if ($field_popsu_themloc_comparatif = $row->getSourceProperty('field_popsu_themloc_comparatif')) {
      // Drush::output()->writeln(dump($field_popsu_themloc_comparatif));
      foreach ($field_popsu_themloc_comparatif as $key => $value) {
        $field_themes_lies[] = $value;
      }
    }
    if (!empty($field_themes_lies)) {
      // Drush::output()->writeln(dump($field_themes_lies));
      $row->setSourceProperty('field_themes_lies', $field_themes_lies);
    }
    // TODO: missing popsu_villes popsu europe ... ?
    if (!empty($field_projets_lies)) {
      // Drush::output()->writeln(dump($field_themes_lies));
      $row->setSourceProperty('field_projets_lies', $field_projets_lies);
    }

    # Text paragraphe
    $text_fields = array(
      'field_popsu_themtrans_body' => 'Texte d\'introduction court',
      'field_popsu_themtrans_introlong' => 'Texte d\'introduction long',
      'field_popsu_themloc_body' => 'Texte de présentation du thème',
      'field_popsu_themeur_titleshort' => 'Version courte de l’intitulé',
      'field_popsu_themeur_title_format' => 'Version formatée de l’intitulé',
      'field_popsu_themeur_intro' => 'texted’introduction',
      'field_popsu_themloc_demarche' => 'Démarche & méthodologie',
      'field_popsu_themtrans_reponses' => 'Réponsesauquestionnaire'
    );
    $textes_paragraphes = array();
    foreach ($text_fields as $key => $title) {
      if ($text = $row->getSourceProperty($key)){
        $text[0]['format'] = 'wysiwyg';
        $p = Paragraph::create(['type' => 'textes']);
        $p->set('field_texte', $text)
        ->set('field_titre', $title)
        ->isNew();
        $p->save();

        $textes_paragraphes[] = array(
          'target_id' => $p->id(),
          'target_revision_id' => $p->getRevisionId(),
        );
      }
    }
    if (!empty($textes_paragraphes)) {
      $row->setSourceProperty('field_textes', $textes_paragraphes);
    }

    return parent::prepareRow($row);
  }

  /**
   * {@inheritdoc}
   */
  public function fields() {
    $fields = [
      'nid' => $this->t('Node ID'),
      'type' => $this->t('Type'),
      'title' => $this->t('Title'),
      'node_uid' => $this->t('Node authored by (uid)'),
      'revision_uid' => $this->t('Revision authored by (uid)'),
      'created' => $this->t('Created timestamp'),
      'changed' => $this->t('Modified timestamp'),
      'status' => $this->t('Published'),
      'promote' => $this->t('Promoted to front page'),
      'sticky' => $this->t('Sticky at top of lists'),
      'revision' => $this->t('Create new revision'),
      'language' => $this->t('Language (fr, en, ...)'),
      'tnid' => $this->t('The translation set id for this node'),
      'timestamp' => $this->t('The timestamp the latest revision of this node was created.'),
    ];
    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function getIds() {
    $ids['nid']['type'] = 'integer';
    $ids['nid']['alias'] = 'n';
    return $ids;
  }

}
